diff --exclude=CVS -ur u-boot-cvs/net/tftp.c u-boot/net/tftp.c
--- u-boot-cvs/net/tftp.c	2003-11-17 16:14:41.000000000 -0500
+++ u-boot/net/tftp.c	2004-02-20 21:09:09.000000000 -0500
@@ -10,8 +10,6 @@
 #include "tftp.h"
 #include "bootp.h"
 
-#undef	ET_DEBUG
-
 #if (CONFIG_COMMANDS & CFG_CMD_NET)
 
 #define WELL_KNOWN_PORT	69		/* Well known TFTP port #		*/
@@ -60,20 +58,48 @@
 {
 	ulong offset = block * 512, newsize = offset + len;
 #ifdef CFG_DIRECT_FLASH_TFTP
-	int i, rc = 0;
+	int i, start_bank, stop_bank;
 
-	for (i=0; i<CFG_MAX_FLASH_BANKS; i++) {
+	start_bank = stop_bank = -1;
+	for (i=0; i<CFG_MAX_FLASH_BANKS; i++) 
+	{
 		/* start address in flash? */
-		if (load_addr + offset >= flash_info[i].start[0]) {
-			rc = 1;
-			break;
+		if ( ((load_addr + offset) >= flash_info[i].start[0]) &&
+		     ((load_addr + offset) < (flash_info[i].start[0] + flash_info[i].size)) )
+		{
+			start_bank = i;
 		}
+		/* stop address in flash? */
+		if ( ((load_addr + newsize) >= flash_info[i].start[0]) &&
+		     ((load_addr + newsize) < (flash_info[i].start[0] + flash_info[i].size)) )
+		{
+			stop_bank = i;
+		}
+		/* can we skip out now? */
+		if ((stop_bank >= 0) && (start_bank >= 0))
+			break;
 	}
 
-	if (rc) { /* Flash is destination for this packet */
-		rc = flash_write ((uchar *)src, (ulong)(load_addr+offset), len);
-		if (rc) {
-			flash_perror (rc);
+	if ((start_bank >=0) || (stop_bank >= 0))
+	{ 
+		/* Flash is destination for this packet */
+		if ( ((start_bank >= 0) && (stop_bank == -1)) ||
+		     ((start_bank >= 0) && (stop_bank == -1)) )
+		{
+			printf("TFTP: FAILURE: write-to-flash start bank %d, stop bank %d\n",
+				start_bank, stop_bank);
+			printf("\tCROSSING THE FLASH BOUNDARY, USE DIFFERENT ADDRESSES\n");
+			printf("\tstart @ 0x%08lx, stop @ 0x%08lx\n", 
+				load_addr + offset, load_addr + newsize);
+			printf("\tblock %d, src = 0x%p, len 0x%x\n",
+				block, src, len);
+			NetState = NETLOOP_FAIL;
+			return;
+		}
+		i = flash_write ((uchar *)src, (ulong)(load_addr+offset), len);
+		if (i) 
+		{
+			flash_perror (i);
 			NetState = NETLOOP_FAIL;
 			return;
 		}
@@ -81,7 +107,28 @@
 	else
 #endif /* CFG_DIRECT_FLASH_TFTP */
 	{
+#if 0 /* def __LITTLE_ENDIAN */
+		unsigned int cnt;
+		unsigned int * val, *tgt;
+		
+		if ((len % sizeof(unsigned int)) != 0)
+			printf("TFTP store block: problem: pkt data not a multiple of %d\n",
+				sizeof(unsigned int));
+		val = (unsigned int*)src;
+		tgt = (unsigned int*)(load_addr + offset);
+		if ((unsigned)val & 0x3)
+			printf("TFTP store block: problem: pkt data ptr not on 4-byte boundary\n");
+		if ((unsigned)tgt & 0x3)
+			printf("TFTP store block: problem: tgt mem ptr not on 4-byte boundary\n");
+		for (cnt=0; cnt<(len/sizeof(unsigned int)); cnt++)
+		{
+			*tgt = __swab32(*val);
+			val++;
+			tgt++;
+		}
+#else
 		(void)memcpy((void *)(load_addr + offset), src, len);
+#endif		
 	}
 
 	if (NetBootFileXferSize < newsize)
